"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ControlWithPropertiesAbstract = exports.PropertyUpdateType = void 0;
const rxjs_1 = require("rxjs");
const crpc_dialog_1 = require("../model/crpc-dialog");
const get_property_1 = require("../protocol/browser/packet/get-property");
const register_event_1 = require("../protocol/packet/register-event");
const status_msg_response_menu_1 = require("../protocol/packet/status-msg-response-menu");
const deferred_rejection_message_1 = require("../util/deferred/deferred-rejection-message");
var PropertyUpdateType;
(function (PropertyUpdateType) {
    PropertyUpdateType[PropertyUpdateType["VALUE_CHANGE"] = 0] = "VALUE_CHANGE";
    PropertyUpdateType[PropertyUpdateType["VALUE_RECEIVE"] = 1] = "VALUE_RECEIVE";
})(PropertyUpdateType = exports.PropertyUpdateType || (exports.PropertyUpdateType = {}));
class ControlWithPropertiesAbstract {
    constructor(_handle, _instanceName, _protocol) {
        this._handle = _handle;
        this._instanceName = _instanceName;
        this._protocol = _protocol;
        this._propertySubjects = {};
        this._propertyValues = {};
        this._propertyUpdateType = {};
        this.busy$ = new rxjs_1.BehaviorSubject(false);
        this.dialog$ = new rxjs_1.Subject();
        this._protocol.event
            .pipe((0, rxjs_1.filter)((event) => event.method === `${this._instanceName}.Event`))
            .subscribe((event) => this._handleEvent(event.params.ev, event.params.parameters));
    }
    getInstanceName() {
        return this._instanceName;
    }
    _onStateChanged(event) {
        Object.keys(event)
            .filter((key) => this._propertySubjects[key])
            .forEach((key) => this._updateLocalProperty(key, event[key]));
    }
    _onBusyChanged(event) {
        const isBusy = true === (event === null || event === void 0 ? void 0 : event.on);
        if (isBusy === this.busy$.getValue()) {
            return;
        }
        if (isBusy) {
            this._busyTimeout = setTimeout(() => this._onBusyChanged(), event === null || event === void 0 ? void 0 : event.timeoutSec);
        }
        else if (this._busyTimeout) {
            clearTimeout(this._busyTimeout);
            delete this._busyTimeout;
        }
        this.busy$.next(isBusy);
    }
    _createPropertySubject(propertyName, defaultValue, updateType = PropertyUpdateType.VALUE_CHANGE, additionalConnectorMethod) {
        this._propertySubjects[propertyName] = new rxjs_1.BehaviorSubject(defaultValue);
        this._propertyUpdateType[propertyName] = updateType;
        return this._propertySubjects[propertyName].pipe((0, rxjs_1.share)({
            connector: () => {
                this._getProperty(propertyName);
                if (additionalConnectorMethod) {
                    additionalConnectorMethod();
                }
                return new rxjs_1.ReplaySubject(1);
            },
            resetOnRefCountZero: true,
        }));
    }
    _updateLocalProperty(propertyName, value) {
        // Property has no subject
        if (undefined === this._propertySubjects[propertyName]) {
            return;
        }
        // If value did not change and update type is VALUE_CHANGE we don't notify the subjects.
        if (this._propertyUpdateType[propertyName] === PropertyUpdateType.VALUE_CHANGE
            && this._propertyValues === value) {
            return;
        }
        this._propertyValues[propertyName] = value;
        this._propertySubjects[propertyName].next(value);
    }
    /**
     * @returns Promise<boolean> true if success, false if error
     */
    async _registerEvent(type) {
        const response = await this._protocol.send(new register_event_1.RegisterEvent(this._handle, type, this._instanceName));
        if (response instanceof deferred_rejection_message_1.DeferredRejectionMessage) {
            console.error(`Failed getting registering ${this._instanceName} event`, type, response.data);
            return false;
        }
        if (response.error !== null) {
            console.error(`${this._instanceName} _registerEvent();`, response.error);
            return false;
        }
        return true;
    }
    async _getProperty(propertyName) {
        const response = await this._protocol.send(new get_property_1.GetProperty(this._instanceName, propertyName));
        if (response instanceof deferred_rejection_message_1.DeferredRejectionMessage) {
            console.error(`Failed getting ${this._instanceName} property event`, response.reason, response.data);
            return;
        }
        Object.keys(response.result)
            .filter((key) => this._propertySubjects[key])
            .forEach((key) => this._updateLocalProperty(key, response.result[key]));
    }
    _onDialogChanged(event) {
        const dialog = Object.assign(new crpc_dialog_1.CrpcDialog((itemNumber, userInput) => {
            this._protocol.send(new status_msg_response_menu_1.StatusMsgResponseMenu(this._instanceName, itemNumber, userInput));
        }), event);
        this.dialog$.next(dialog);
    }
}
exports.ControlWithPropertiesAbstract = ControlWithPropertiesAbstract;
