"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CrpcContainer = void 0;
const crpc_object_1 = require("../model/crpc-object");
const crpc_1 = require("../protocol/crpc");
const event_name_1 = require("../protocol/event-name");
const get_objects_1 = require("../protocol/packet/get-objects");
const register_1 = require("../protocol/packet/register");
const register_event_1 = require("../protocol/packet/register-event");
const get_menu_1 = require("../protocol/player/packet/get-menu");
const deferred_rejection_message_1 = require("../util/deferred/deferred-rejection-message");
const browser_1 = require("./browser");
const player_1 = require("./player");
class CrpcContainer {
    /**
     * @param _uuid Make sure this id is always the same for the client you are using!
     * @param _handle Possible custom handle to use for comms, by default crestron used "sg", we use "hmp(Html Media Player)"
     */
    constructor(_uuid, _handle = 'hmp') {
        this._uuid = _uuid;
        this._handle = _handle;
        this._objects = [];
        this._protocol = new crpc_1.CrpcProtocol(_handle);
    }
    async initialize() {
        await this.register();
        await this._getObjects();
        await this._registerEvent(event_name_1.EventName.ObjectDirectoryChanged);
    }
    pipeFromServer(data) {
        this._protocol.receive(data);
    }
    get pipeToServer() {
        return this._protocol.transmit;
    }
    async register() {
        await this._protocol.send(new register_1.Register(this._uuid, false, this._protocol.version));
    }
    async _getObjects() {
        const response = await this._protocol.send(new get_objects_1.GetObjects());
        if (response instanceof deferred_rejection_message_1.DeferredRejectionMessage) {
            console.error('Failed getting objects', response.data);
            return;
        }
        this._objects = response.result.objects.object.map((rawObj) => {
            const obj = Object.assign(new crpc_object_1.CrpcObject(), rawObj);
            if (obj.isMediaPlayer()) {
                this.player = new player_1.CrpcPlayer(this._handle, obj.uuid, obj.instanceName, this._protocol);
            }
            if (obj.isMenu() && obj.uuid === this._uuid) {
                this.browser = new browser_1.CrpcBrowser(this._handle, obj.uuid, obj.instanceName, this._protocol);
            }
            return obj;
        });
        if (!this.browser) {
            this.browser = await this._getBrowser();
        }
    }
    /**
     * @returns Promise<CrpcBrowser | undefined> if undefined the browser could not be retrieved for some reason!
     */
    async _getBrowser() {
        if (!this.player) {
            throw Error('Could not get CrpcBrowser as there is no player available to browse!');
        }
        if (this.browser) {
            return this.browser;
        }
        const response = await this._protocol.send(new get_menu_1.GetMenu(this.player.getInstanceName(), this._uuid));
        if (response instanceof deferred_rejection_message_1.DeferredRejectionMessage) {
            console.error('Failed getting objects', response.data);
            return undefined;
        }
        const instanceName = response.result.instanceName || response.result.instancename;
        return new browser_1.CrpcBrowser(this._handle, this._uuid, instanceName, this._protocol);
    }
    /**
     * @returns Promise<boolean> true if success, false if error
     */
    async _registerEvent(type) {
        const response = await this._protocol.send(new register_event_1.RegisterEvent(this._handle, type));
        if (response instanceof deferred_rejection_message_1.DeferredRejectionMessage) {
            console.error('Failed getting registering event ', type, response.data);
            return false;
        }
        if (response.error !== null) {
            console.error('registerEvent(); ', response.error);
            return false;
        }
        return true;
    }
}
exports.CrpcContainer = CrpcContainer;
