"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CrpcProtocol = void 0;
const rxjs_1 = require("rxjs");
const crpc_event_packet_1 = require("../model/crpc-event-packet");
const receive_packet_1 = require("../model/receive-packet");
const transmit_packet_1 = require("../model/transmit-packet");
const temporary_deferred_1 = require("../util/deferred/temporary-deferred");
class CrpcProtocol {
    constructor(_handle) {
        this._handle = _handle;
        /**
         *  @description Output to inject into transport
         */
        this.transmit = new rxjs_1.Subject();
        /**
         * @description Once an event is received it will pass through here.
         */
        this.event = new rxjs_1.Subject();
        this.dialog = new rxjs_1.Subject();
        this.version = '1.0';
        this._jsonStart = '{"jsonrpc":';
        this._deferredById = {};
        this._lastMessageId = Math.floor(Math.random() * (65535 - 10 + 1)) + 10;
        this._parseBuffer = '';
        this._lastParsedInput = '';
    }
    _parsePartial(input) {
        // Make sure to never inject double messages into our buffer.
        if (this._lastParsedInput === input) {
            console.log('Received duplicate input for parsing, ignored it.');
            return;
        }
        this._lastParsedInput = input;
        const header = input.substring(0, 8);
        const isEnd = header.charAt(3) === 'e';
        const partialMessage = input.substring(8);
        // If message is not complete, start filling up the buffer.
        if (!isEnd) {
            // When we receive
            if (partialMessage.startsWith(this._jsonStart) &&
                this._parseBuffer !== '') {
                this._parseBuffer = partialMessage;
                console.warn('Buffer already had data but received new json start, clearing!');
                return;
            }
            this._parseBuffer = this._parseBuffer + partialMessage;
            return;
        }
        // Message end received, start parsing
        this._parseFull(this._parseBuffer + partialMessage);
        this._parseBuffer = '';
    }
    _parseFull(data) {
        const messages = data.split(this._jsonStart);
        if (messages.length > 2) {
            messages.forEach((message) => {
                if (message === '') {
                    return true;
                }
                this._parseFull(this._jsonStart + message);
            });
            return;
        }
        let packet = new receive_packet_1.ReceivePacket();
        try {
            packet = Object.assign(packet, JSON.parse(data));
        }
        catch (ex) {
            console.error(new Date(), 'Failed parsing json;', ex);
            return;
        }
        this._handlePacket(packet);
    }
    _handlePacket(packet) {
        var _a, _b;
        // Check for callbacks and call them when available
        if (packet.id && this._deferredById[packet.id]) {
            if (packet.error) {
                (_a = this._deferredById[packet.id]) === null || _a === void 0 ? void 0 : _a.reject(packet);
                delete this._deferredById[packet.id];
            }
            else {
                (_b = this._deferredById[packet.id]) === null || _b === void 0 ? void 0 : _b.resolve(packet);
                delete this._deferredById[packet.id];
            }
        }
        // Check for event type messages to trigger event subject
        if (packet.method && packet.method.endsWith('.Event')) {
            const event = Object.assign(new crpc_event_packet_1.CrpcEventPacket(), packet);
            this.event.next(event);
            return;
        }
    }
    receive(message) {
        this._parsePartial(message);
    }
    sendRaw(instanceStr, method, params = null) {
        const messageId = this._lastMessageId >= 65535 ? 1000 : this._lastMessageId + 1;
        const message = Object.assign(new transmit_packet_1.TransmitPacket(), {
            id: messageId,
            method: `${instanceStr}.${method}`,
            params,
        });
        let data = JSON.stringify(message);
        while (data.length > 0) {
            const partial = data.substring(0, data.length > 247 ? 247 : data.length);
            data = data.replace(partial, '');
            const flag = data.length === 0 ? 'e' : 'c';
            this.transmit.next(`205${flag}00${partial.length.toString(16)}${partial}`);
        }
        this._deferredById[messageId] = new temporary_deferred_1.TemporaryDeferred(10000, () => delete this._deferredById[messageId]);
        this._lastMessageId = messageId;
        return this._deferredById[messageId].promise;
    }
    send(packet) {
        return this.sendRaw(packet.instanceName || 'Crpc', packet.method, packet.body);
    }
}
exports.CrpcProtocol = CrpcProtocol;
